import { Result } from "./Result";
export type Step = ((() => Promise<Result>) | ((prev: Result) => Promise<Result>));
export declare class Try<T> {
    _finalResult?: Result;
    readonly steps: Step[];
    private constructor();
    static of<T>(func: () => T | Promise<T>): Try<T>;
    static combine<T extends any[], R>(...args: [...{
        [K in keyof T]: Try<T[K]>;
    }, (...values: T) => R | Promise<R>]): Try<R>;
    static success<U>(value: U): Try<U>;
    static failure<T>(err: Error): Try<T>;
    map<U>(func: (value: T) => U | Promise<U>): Try<U>;
    mapIf<U>(predicateFunc: (value: T) => boolean | Promise<boolean>, func: (value: T) => U | Promise<U>): Try<U>;
    mapFailureWith<E extends Error, U extends Error>(errorType: new (...args: any[]) => E, func: (ex: E) => U | Promise<U>): Try<T>;
    flatMap<U>(func: (value: T) => Try<U> | Promise<Try<U>>): Try<U>;
    flatMapIf<U>(predicateFunc: (value: T) => boolean | Promise<boolean>, func: (value: T) => Try<U> | Promise<Try<U>>): Try<U>;
    getOrElse<U>(fallbackValue: U): Promise<U | T>;
    getOrElseGet<U>(func: (ex: Error) => U | Promise<U>): Promise<T | U>;
    getOrElseThrow(func: (error: Error) => Promise<Error> | Error): Promise<T>;
    peek(func: (value: T) => Promise<void> | void): Try<T>;
    andThen(func: (value: T) => Promise<void> | void): Try<T>;
    filter(predicateFunc: (value: T) => boolean | Promise<boolean>, errorProvider?: (value: T) => Error): Try<T>;
    filterNot(predicateFunc: (value: T) => boolean | Promise<boolean>, errorProvider?: (value: T) => Error): Try<T>;
    recover<U>(func: (error: Error) => U | Promise<U>): Try<T | U>;
    recoverWith<U>(func: (error: Error) => Try<U> | Promise<Try<U>>): Try<U | T>;
    get(): Promise<T>;
    run(): Promise<Try<T>>;
    isSuccess(): boolean;
    isFailure(): boolean;
    andFinally(func: () => Promise<void> | void): Try<T>;
    mapFailure(func: (ex: Error) => Error | Promise<Error>): Try<T>;
    getCause(): Error | undefined;
    onSuccess(func: (value: T) => Promise<void> | void): Try<T>;
    onFailure(func: (value: Error) => Promise<void> | void): Try<T>;
}
