"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Try = void 0;
const functions_1 = require("./functions");
class Try {
    constructor(steps = []) {
        this._finalResult = undefined;
        this.steps = [];
        this.steps = steps;
    }
    static of(func) {
        return new Try([() => (0, functions_1.of)(func)]);
    }
    static combine(...args) {
        // @ts-ignore
        return new Try([() => (0, functions_1.combine)(...args)]);
    }
    static success(value) {
        return new Try([() => (0, functions_1.success)(value)]);
    }
    static failure(err) {
        return new Try([() => (0, functions_1.failure)(err)]);
    }
    map(func) {
        return new Try([...this.steps, (prev) => (0, functions_1.map)(prev, func)]);
    }
    mapIf(predicateFunc, func) {
        return new Try([...this.steps, (prev) => (0, functions_1.mapIf)(prev, predicateFunc, func)]);
    }
    mapFailureWith(errorType, func) {
        return new Try([...this.steps, (prev) => (0, functions_1.mapFailureWith)(prev, errorType, func)]);
    }
    flatMap(func) {
        return new Try([...this.steps, (prev) => (0, functions_1.flatMap)(prev, func)]);
    }
    flatMapIf(predicateFunc, func) {
        return new Try([...this.steps, (prev) => (0, functions_1.flatMapIf)(prev, predicateFunc, func)]);
    }
    getOrElse(fallbackValue) {
        return __awaiter(this, void 0, void 0, function* () {
            return (0, functions_1.getOrElse)(this, fallbackValue);
        });
    }
    getOrElseGet(func) {
        return __awaiter(this, void 0, void 0, function* () {
            return (0, functions_1.getOrElseGet)(this, func);
        });
    }
    getOrElseThrow(func) {
        return __awaiter(this, void 0, void 0, function* () {
            return (0, functions_1.getOrElseThrow)(this, func);
        });
    }
    peek(func) {
        //According to the docs, peek is the same as `andThen`
        return new Try([...this.steps, (prev) => (0, functions_1.andThen)(prev, func)]);
    }
    andThen(func) {
        return new Try([...this.steps, (prev) => (0, functions_1.andThen)(prev, func)]);
    }
    filter(predicateFunc, errorProvider) {
        return new Try([...this.steps, (prev) => (0, functions_1.filter)(prev, predicateFunc, errorProvider)]);
    }
    filterNot(predicateFunc, errorProvider) {
        return new Try([...this.steps, (prev) => (0, functions_1.filterNot)(prev, predicateFunc, errorProvider)]);
    }
    recover(func) {
        return new Try([...this.steps, (prev) => (0, functions_1.recover)(prev, func)]);
    }
    recoverWith(func) {
        return new Try([...this.steps, (prev) => (0, functions_1.recoverWith)(prev, func)]);
    }
    get() {
        return __awaiter(this, void 0, void 0, function* () {
            return (0, functions_1.get)(this);
        });
    }
    run() {
        return __awaiter(this, void 0, void 0, function* () {
            yield (0, functions_1.run)(this);
            return this;
        });
    }
    isSuccess() {
        return !this._finalResult.isError();
    }
    isFailure() {
        return this._finalResult.isError();
    }
    andFinally(func) {
        return new Try([...this.steps, (prev) => (0, functions_1.andFinally)(prev, func)]);
    }
    mapFailure(func) {
        return new Try([...this.steps, (prev) => (0, functions_1.mapFailure)(prev, func)]);
    }
    getCause() {
        return this._finalResult.getError();
    }
    onSuccess(func) {
        return new Try([...this.steps, (prev) => (0, functions_1.onSuccess)(prev, func)]);
    }
    onFailure(func) {
        return new Try([...this.steps, (prev) => (0, functions_1.onFailure)(prev, func)]);
    }
}
exports.Try = Try;
